@@ -1,14 +1,17 @@
 import json
 import tensorflow as tf
-from object_detection.utils import dataset_util
-from object_detection.utils import config_util
-from object_detection.utils import label_map_util
 from PIL import Image, ImageDraw
 import os
 import numpy as np 
 import cv2
 import functools
 import sys
+from IPython.display import display
+import random
+
+from object_detection.utils import dataset_util
+from object_detection.utils import config_util
+from object_detection.utils import label_map_util
 from tensorboard.backend.event_processing.event_accumulator import EventAccumulator
 from object_detection.builders import dataset_builder
 from object_detection.builders import graph_rewriter_builder
@@ -18,8 +21,11 @@ from object_detection.legacy import evaluator
 from object_detection import exporter
 from object_detection.utils import ops as utils_ops
 from object_detection.utils import visualization_utils as vis_util
-from IPython.display import display
-import random
+from object_detection import exporter_lib_v2
+from object_detection.protos import pipeline_pb2
+from object_detection import model_lib_v2
+from tensorflow.python.summary.summary_iterator import summary_iterator
+from google.protobuf import text_format
 
 
 def create_record_files(dict_annotations, train_list, train_list_id, eval_list, 
@@ -42,7 +48,7 @@ def create_record_files(dict_annotations, train_list, train_list_id, eval_list,
     
     for dataset in datasets:
         output_path = os.path.join(record_dir, dataset+".record")
-        writer = tf.python_io.TFRecordWriter(output_path)
+        writer = tf.compat.v1.python_io.TFRecordWriter(output_path)
         print(f"Creating record file at {output_path}")
         for variables in tfExample_generator(dict_annotations, train_list, train_list_id, eval_list, 
                             eval_list_id, label_map, ensemble=dataset, annotation_type=annotation_type):
@@ -326,108 +332,121 @@ def edit_config(model_selected, config_output_dir, num_steps, label_map_path, re
     print(f"Configuration successfully edited and saved at {config_output_dir}")
 
 
+def train(ckpt_dir='', config_path='', train_steps=1000, use_tpu=False, checkpoint_every_n=100,
+          record_summaries=True):
 
-def train(master='', save_summaries_secs=30, task=0, num_clones=1, clone_on_cpu=False, worker_replicas=1, ps_tasks=0, 
-                    ckpt_dir='', conf_dir='', train_config_path='', input_config_path='', model_config_path=''):   
+    tf.config.set_soft_device_placement(True)
+    strategy = tf.compat.v2.distribute.MirroredStrategy()
+
+    with strategy.scope():
+      model_lib_v2.train_loop(
+          pipeline_config_path=config_path,
+          model_dir=ckpt_dir,
+          train_steps=train_steps,
+          use_tpu=use_tpu,
+          checkpoint_every_n=checkpoint_every_n,
+          record_summaries=record_summaries)
+# def train(master='', save_summaries_secs=30, task=0, num_clones=1, clone_on_cpu=False, worker_replicas=1, ps_tasks=0, 
+#                     ckpt_dir='', conf_dir='', train_config_path='', input_config_path='', model_config_path=''):   
    
 
-    pipeline_config_path = os.path.join(conf_dir,"pipeline.config")
-    configs = config_util.get_configs_from_pipeline_file(pipeline_config_path)
+#     pipeline_config_path = os.path.join(conf_dir,"pipeline.config")
+#     configs = config_util.get_configs_from_pipeline_file(pipeline_config_path)
 
-    tf.logging.set_verbosity(tf.logging.INFO)
-    assert ckpt_dir, '`ckpt_dir` is missing.'
-    if task == 0: tf.gfile.MakeDirs(ckpt_dir)
-    if pipeline_config_path:
-        configs = config_util.get_configs_from_pipeline_file(pipeline_config_path)
-        if task == 0:
-            tf.gfile.Copy(pipeline_config_path,
-                          os.path.join(ckpt_dir, 'pipeline.config'),
-                          overwrite=True)
-    else:
-        configs = config_util.get_configs_from_multiple_files(
-                        model_config_path=model_config_path,
-                        train_config_path=train_config_path,
-                        train_input_config_path=input_config_path)
-        if task == 0:
-            for name, config in [('model.config', model_config_path),
-                                ('train.config', train_config_path),
-                                ('input.config', input_config_path)]:
-                tf.gfile.Copy(config, os.path.join(ckpt_dir, name),
-                            overwrite=True)
+#     tf.compat.v1.logging.set_verbosity(tf.compat.v1.logging.INFO)
+#     assert ckpt_dir, '`ckpt_dir` is missing.'
+#     if task == 0: tf.compat.v1.gfile.MakeDirs(ckpt_dir)
+#     if pipeline_config_path:
+#         configs = config_util.get_configs_from_pipeline_file(pipeline_config_path)
+#         if task == 0:
+#             tf.compat.v1.gfile.Copy(pipeline_config_path,
+#                           os.path.join(ckpt_dir, 'pipeline.config'),
+#                           overwrite=True)
+#     else:
+#         configs = config_util.get_configs_from_multiple_files(
+#                         model_config_path=model_config_path,
+#                         train_config_path=train_config_path,
+#                         train_input_config_path=input_config_path)
+#         if task == 0:
+#             for name, config in [('model.config', model_config_path),
+#                                 ('train.config', train_config_path),
+#                                 ('input.config', input_config_path)]:
+#                 tf.compat.v1.gfile.Copy(config, os.path.join(ckpt_dir, name),
+#                             overwrite=True)
 
-    model_config = configs['model']
-    train_config = configs['train_config']
-    input_config = configs['train_input_config']
+#     model_config = configs['model']
+#     train_config = configs['train_config']
+#     input_config = configs['train_input_config']
 
-    model_fn = functools.partial(
-        model_builder.build,
-        model_config=model_config,
-        is_training=True)
+#     model_fn = functools.partial(
+#         model_builder.build,
+#         model_config=model_config,
+#         is_training=True)
 
-    def get_next(config):
-        return dataset_builder.make_initializable_iterator(
-            dataset_builder.build(config)).get_next()
+#     def get_next(config):
+#         return dataset_builder.make_initializable_iterator(
+#             dataset_builder.build(config)).get_next()
 
-    create_input_dict_fn = functools.partial(get_next, input_config)
+#     create_input_dict_fn = functools.partial(get_next, input_config)
 
-    env = json.loads(os.environ.get('TF_CONFIG', '{}'))
-    cluster_data = env.get('cluster', None)
-    cluster = tf.train.ClusterSpec(cluster_data) if cluster_data else None
-    task_data = env.get('task', None) or {'type': 'master', 'index': 0}
-    task_info = type('TaskSpec', (object,), task_data)
+#     env = json.loads(os.environ.get('TF_CONFIG', '{}'))
+#     cluster_data = env.get('cluster', None)
+#     cluster = tf.train.ClusterSpec(cluster_data) if cluster_data else None
+#     task_data = env.get('task', None) or {'type': 'master', 'index': 0}
+#     task_info = type('TaskSpec', (object,), task_data)
 
-    # Parameters for a single worker.
-    ps_tasks = 0
-    worker_replicas = 1
-    worker_job_name = 'lonely_worker'
-    task = 0
-    is_chief = True
-    master = ''
+#     # Parameters for a single worker.
+#     ps_tasks = 0
+#     worker_replicas = 1
+#     worker_job_name = 'lonely_worker'
+#     task = 0
+#     is_chief = True
+#     master = ''
 
-    if cluster_data and 'worker' in cluster_data:
-    # Number of total worker replicas include "worker"s and the "master".
-        worker_replicas = len(cluster_data['worker']) + 1
-    if cluster_data and 'ps' in cluster_data:
-        ps_tasks = len(cluster_data['ps'])
+#     if cluster_data and 'worker' in cluster_data:
+#     # Number of total worker replicas include "worker"s and the "master".
+#         worker_replicas = len(cluster_data['worker']) + 1
+#     if cluster_data and 'ps' in cluster_data:
+#         ps_tasks = len(cluster_data['ps'])
 
-    if worker_replicas > 1 and ps_tasks < 1:
-        raise ValueError('At least 1 ps task is needed for distributed training.')
+#     if worker_replicas > 1 and ps_tasks < 1:
+#         raise ValueError('At least 1 ps task is needed for distributed training.')
 
-    if worker_replicas >= 1 and ps_tasks > 0:
-    # Set up distributed training.
-        server = tf.train.Server(tf.train.ClusterSpec(cluster), protocol='grpc',
-                                job_name=task_info.type,
-                                task_index=task_info.index)
-        if task_info.type == 'ps':
-            server.join()
-            return
+#     if worker_replicas >= 1 and ps_tasks > 0:
+#     # Set up distributed training.
+#         server = tf.train.Server(tf.train.ClusterSpec(cluster), protocol='grpc',
+#                                 job_name=task_info.type,
+#                                 task_index=task_info.index)
+#         if task_info.type == 'ps':
+#             server.join()
+#             return
 
-        worker_job_name = '%s/task:%d' % (task_info.type, task_info.index)
-        task = task_info.index
-        is_chief = (task_info.type == 'master')
-        master = server.target
+#         worker_job_name = '%s/task:%d' % (task_info.type, task_info.index)
+#         task = task_info.index
+#         is_chief = (task_info.type == 'master')
+#         master = server.target
 
-    graph_rewriter_fn = None
-    if 'graph_rewriter_config' in configs:
-        graph_rewriter_fn = graph_rewriter_builder.build(
-            configs['graph_rewriter_config'], is_training=True)
+#     graph_rewriter_fn = None
+#     if 'graph_rewriter_config' in configs:
+#         graph_rewriter_fn = graph_rewriter_builder.build(
+#             configs['graph_rewriter_config'], is_training=True)
 
-    trainer.train(
-        create_input_dict_fn,
-        model_fn,
-        train_config,
-        master,
-        task,
-        num_clones,
-        worker_replicas,
-        clone_on_cpu,
-        ps_tasks,
-        worker_job_name,
-        is_chief,
-        ckpt_dir,
-        graph_hook_fn=graph_rewriter_fn)
+#     trainer.train(
+#         create_input_dict_fn,
+#         model_fn,
+#         train_config,
+#         master,
+#         task,
+#         num_clones,
+#         worker_replicas,
+#         clone_on_cpu,
+#         ps_tasks,
+#         worker_job_name,
+#         is_chief,
+#         ckpt_dir,
+#         graph_hook_fn=graph_rewriter_fn)
 
-def evaluate(eval_dir, config_dir, checkpoint_dir, eval_training_data=False):
+def evaluate(metrics_dir='', config_path='', ckpt_dir=''):
     '''
         Function used to evaluate your trained model. 
 
@@ -443,46 +462,18 @@ def evaluate(eval_dir, config_dir, checkpoint_dir, eval_training_data=False):
         Returns:
             A dictionnary of metrics ready to be sent to the picsell.ia platform.
     '''
-
-    tf.reset_default_graph()
-    tf.gfile.MakeDirs(eval_dir)
-    configs = config_util.get_configs_from_pipeline_file(os.path.join(config_dir, "pipeline.config"))
-    model_config = configs['model']
-    eval_config = configs['eval_config']
-    input_config = configs['eval_input_config']
-    if eval_training_data:
-        input_config = configs['train_input_config']
-
-    model_fn = functools.partial(model_builder.build, model_config=model_config, is_training=False)
-
-    def get_next(config):
-        return dataset_builder.make_initializable_iterator(dataset_builder.build(config)).get_next()
-
-    create_input_dict_fn = functools.partial(get_next, input_config)
-
-    categories = label_map_util.create_categories_from_labelmap(
-      input_config.label_map_path)
-
-    eval_config.max_evals = 1
-
-    graph_rewriter_fn = None
-    if 'graph_rewriter_config' in configs:
-        graph_rewriter_fn = graph_rewriter_builder.build(
-            configs['graph_rewriter_config'], is_training=False)
-
-    metrics = evaluator.evaluate(
-          create_input_dict_fn,
-          model_fn,
-          eval_config,
-          categories,
-          checkpoint_dir,
-          eval_dir,
-          graph_hook_fn=graph_rewriter_fn)
-    return {k:str(round(v, 3)) for k,v in metrics.items()}
+    tf.config.set_soft_device_placement(True)
+    model_lib_v2.eval_continuously(
+        pipeline_config_path=config_path,
+        model_dir=metrics_dir,
+        train_steps=None,
+        sample_1_of_n_eval_examples=1,
+        sample_1_of_n_eval_on_train_examples=(1),
+        checkpoint_dir=ckpt_dir,
+        wait_interval=4, timeout=5)
 
 
-
-def tfevents_to_dict(path):
+def tf_events_to_dict(path, type=''):
     '''Get a dictionnary of scalars from the tfevent inside the training directory.
 
         Args: 
@@ -491,37 +482,27 @@ def tfevents_to_dict(path):
         Returns:
             A dictionnary of scalars logs.
     '''
-    event = [filename for filename in os.listdir(path) if filename.startswith("events.out")][-1]
-    event_acc = EventAccumulator(os.path.join(path,event)).Reload()
-    logs = dict()
-    # dict_logs = dict()
-    # K=0
-    for scalar_key in event_acc.scalars.Keys():
-        scalar_dict = {"step": [], "value": []}
+    log_dict = {}
+    for summary in summary_iterator(path):
+        for v in summary.summary.value:
+            if not 'image' in v.tag:
+                if v.tag in log_dict.keys():
+                    decoded = tf.compat.v1.decode_raw(v.tensor.tensor_content, tf.float32)
+                    log_dict[v.tag]["step"].append(str(len(log_dict[v.tag]["step"])+1))
+                    log_dict[v.tag]["value"].append(str(tf.cast(decoded, tf.float32).numpy()[0]))
+                else:
+                    decoded = tf.compat.v1.decode_raw(v.tensor.tensor_content, tf.float32)
+                    if type=='train':
+                        scalar_dict = {"step": [0], "value": [str(tf.cast(decoded, tf.float32).numpy()[0])]}
+                        log_dict[v.tag] = scalar_dict
+                    if type=='eval':
+                        log_dict[v.tag] = str(tf.cast(decoded, tf.float32).numpy()[0])
 
-        for scalars in event_acc.Scalars(scalar_key):
-            scalar_dict["step"].append(scalars.step)
-            scalar_dict["value"].append(scalars.value)
-        logs[scalar_key] = scalar_dict
-    #     if K==0:
-    #         K = len(event_acc.Scalars(scalar_key))
-    #         if K<500:
-    #             Idx = np.logspace(1,np.log10(K-1), int(K/10), dtype=np.uint32)
-    #             Idx = list(np.arange(0,9,1))+list(Idx)
+    return log_dict
 
-    # if len(Idx)<=len(logs["Losses/TotalLoss"]["value"]):
-    #     for key in logs.keys():
-    #         dict_logs[key]=dict.fromkeys(["value","step"])
-    #         dict_logs[key]["value"] = list(np.array(logs[key]["value"])[Idx])
-    #         dict_logs[key]["step"] = list(np.array(logs[key]["step"])[Idx].astype(float))
-    #     return dict_logs
-
-    return logs
-    
-
-def export_infer_graph(ckpt_dir, exported_model_dir, pipeline_config_path,
-                        write_inference_graph=False, input_type="image_tensor", input_shape=None, sufix=True):
-    
+def export_graph(ckpt_dir, exported_model_dir, pipeline_config_dir,
+    config_override='', input_type="image_tensor", use_side_inputs=False,
+    side_input_shapes='', side_input_types='', side_input_names=''):
     ''' Export your checkpoint to a saved_model.pb file
         Args:
             Required:
@@ -530,26 +511,18 @@ def export_infer_graph(ckpt_dir, exported_model_dir, pipeline_config_path,
                 pipeline_çonfig_path: The directory where you protobuf configuration is located.
 
     '''
+    pipeline_config_path = os.path.join(pipeline_config_dir,"pipeline.config")
+    pipeline_config = pipeline_pb2.TrainEvalPipelineConfig()
+    with tf.io.gfile.GFile(pipeline_config_path, 'r') as f:
+        text_format.Merge(f.read(), pipeline_config)
+    text_format.Merge(config_override, pipeline_config)
+    exporter_lib_v2.export_inference_graph(
+        input_type, pipeline_config, ckpt_dir,
+        exported_model_dir, use_side_inputs, side_input_shapes,
+        side_input_types, side_input_names)
+        
 
-    tf.reset_default_graph()
-    pipeline_config_path = os.path.join(pipeline_config_path,"pipeline.config")
-    config_dict = config_util.get_configs_from_pipeline_file(pipeline_config_path)
-    ckpt_number = str(config_dict["train_config"].num_steps)
-    pipeline_config = config_util.create_pipeline_proto_from_configs(config_dict)
-    if sufix==False:
-        trained_checkpoint_prefix = os.path.join(ckpt_dir,'model.ckpt')
-    else:
-        trained_checkpoint_prefix = os.path.join(ckpt_dir,'model.ckpt-'+ckpt_number)
-    exporter.export_inference_graph(
-        input_type, pipeline_config, trained_checkpoint_prefix,
-        exported_model_dir, input_shape=input_shape,
-        write_inference_graph=write_inference_graph)
-
-
-
-
-def infer(path, exported_model_dir, label_map_path, results_dir, disp=True, num_infer=5, min_score_thresh=0.7, from_tfrecords=False):
-
+def run_inference(path, exported_model_dir, label_map_path, results_dir, disp=True, num_infer=5, min_score_thresh=0.7, from_tfrecords=False):
     ''' Use your exported model to infer on a path list of images. 
 
         Args:
@@ -566,7 +539,7 @@ def infer(path, exported_model_dir, label_map_path, results_dir, disp=True, num_
 
     '''
     saved_model_path = os.path.join(exported_model_dir, "saved_model")
-    predict_fn = tf.contrib.predictor.from_saved_model(saved_model_path)
+    model = tf.saved_model.load(saved_model_path)
     if not from_tfrecords:
         path_list = path
         random.shuffle(path_list)
@@ -574,83 +547,57 @@ def infer(path, exported_model_dir, label_map_path, results_dir, disp=True, num_
     elif from_tfrecords:
         eval_path = os.path.join(path, "eval.record")
         path_list = []
-        for example in tf.python_io.tf_record_iterator(eval_path):
+        for example in tf.compat.v1.python_io.tf_record_iterator(eval_path):
             tf_examp = tf.train.Example.FromString(example)
             for key in tf_examp.features.feature:
                 if key=="image/filename":
                     path_list.append(tf_examp.features.feature[key].bytes_list.value[0].decode("utf-8"))
     path_list = path_list[:num_infer]
-    with tf.Session() as sess:
-        category_index = label_map_util.create_category_index_from_labelmap(label_map_path)
-        counter=0
-        for img_path in path_list:
-            try:
-                img = cv2.imread(img_path)
-                img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
-                img_tensor = np.expand_dims(img, 0)
-                output_dict = predict_fn({"inputs": img_tensor})
-                
+    category_index = label_map_util.create_category_index_from_labelmap(label_map_path)
+    counter=0
+    for img_path in path_list:
+        try:
+            img = cv2.imread(img_path)
+            img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
+            img_tensor = np.expand_dims(img, 0)
+            result = model(img_tensor)
+            label_id_offset = 0
+            print(result['detection_classes'][0])
+            if 'detection_masks' in result:
+                # we need to convert np.arrays to tensors
+                detection_masks = tf.convert_to_tensor(result['detection_masks'][0])
+                detection_boxes = tf.convert_to_tensor(result['detection_boxes'][0])
+
+                # Reframe the the bbox mask to the image size.
+                detection_masks_reframed = utils_ops.reframe_box_masks_to_image_masks(
+                            detection_masks, detection_boxes,
+                            img.shape[1], img.shape[2])
+                detection_masks_reframed = tf.cast(detection_masks_reframed > 0.5,
+                                                    tf.uint8)
+                result['detection_masks_reframed'] = detection_masks_reframed.numpy()
+
+
+            vis_util.visualize_boxes_and_labels_on_image_array(
+                img,
+                (result['detection_boxes'][0]).numpy(),
+                (result['detection_classes'][0]).numpy().astype(int),
+                (result['detection_scores'][0]).numpy(),
+                category_index,
+                use_normalized_coordinates=True,
+                max_boxes_to_draw=200,
+                min_score_thresh=.30,
+                agnostic_mode=False,
+                instance_masks=result.get('detection_masks_reframed', None),
+                line_thickness=8)
+
+            img_name = img_path.split("/")[-1]
+            Image.fromarray(img).save(os.path.join(results_dir,img_name))
             
+            if disp == True:
+                display(Image.fromarray(img))
 
-                num_detections = int(output_dict.pop('num_detections'))
-                output_dict = {key:value[0, :num_detections] for key,value in output_dict.items()}
-                output_dict['num_detections'] = num_detections
-                output_dict['detection_classes'] = output_dict['detection_classes'].astype(np.int64)
-
-                if 'detection_masks' in output_dict:
-                    # Reframe the the bbox mask to the image size.
-                    detection_masks_reframed = utils_ops.reframe_box_masks_to_image_masks(
-                                                                output_dict['detection_masks'], 
-                                                                output_dict['detection_boxes'],
-                                                                img.shape[0], 
-                                                                img.shape[1])      
-                    detection_masks_reframed = tf.cast(detection_masks_reframed > 0.5, tf.uint8)          
-                    mask_refr = sess.run(detection_masks_reframed)
-                    output_dict['detection_masks_reframed'] = mask_refr
-                masks = output_dict.get('detection_masks_reframed', None)
-                boxes = output_dict["detection_boxes"]
-                classes = output_dict["detection_classes"]
-                scores = output_dict["detection_scores"]
-                
-
-                b = []
-                c = []
-                s = []
-                m = []
-                k = 0
-                for classe in classes:
-                    b.append(boxes[k])
-                    c.append(classe)
-                    s.append(scores[k])
-                    if masks is not None:
-                        m.append(masks[k])
-                    k+=1
-                boxes = np.array(b)
-                classes = np.array(c)
-                scores = np.array(s)
-                if masks is not None:
-                    masks = np.array(m)
-
-
-                vis_util.visualize_boxes_and_labels_on_image_array(img, 
-                                                    boxes,
-                                                    classes,
-                                                    scores,
-                                                    category_index,
-                                                    instance_masks=masks,
-                                                    use_normalized_coordinates=True,
-                                                    line_thickness=3,
-                                                    min_score_thresh=min_score_thresh,
-                                                    max_boxes_to_draw=None)
-
-                img_name = img_path.split("/")[-1]
-                Image.fromarray(img).save(os.path.join(results_dir,img_name))
-                
-                if disp == True:
-                    display(Image.fromarray(img))
-
-            except:
-                counter+=1
-                continue
+        except:
+            counter+=1
+            continue
     if counter>0:
         print(f"{counter} weren't infered")
\ No newline at end of file
